// MIT License
//
// Copyright (c) 2024 vinchent <vinchent@vinchent.xyz>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

package controller

import (
	"net/http"

	"git.vinchent.xyz/vinchent/howmuch/internal/howmuch/model"
	"git.vinchent.xyz/vinchent/howmuch/internal/howmuch/usecase/usecase"
	"git.vinchent.xyz/vinchent/howmuch/internal/pkg/core"
	"git.vinchent.xyz/vinchent/howmuch/internal/pkg/errno"
	"github.com/asaskevich/govalidator"
	"github.com/gin-gonic/gin"
)

// User is the user controller interface, it describes all the handlers
// that need to be implemented for the /user endpoint
type User interface {
	Create(core.Context)
	UpdateInfo(*gin.Context)
	ChangePassword(*gin.Context)
}

type UserController struct {
	userUsecase usecase.User
}

var UserParamsErr = &errno.Errno{
	HTTP:    http.StatusBadRequest,
	Code:    errno.ErrorCode(errno.InvalidParameterCode, "UserParamsErr"),
	Message: "user info is not correct",
}

func NewUserController(us usecase.User) User {
	return &UserController{
		userUsecase: us,
	}
}

func (uc *UserController) Create(ctx core.Context) {
	var params model.User

	if err := ctx.Bind(&params); err != nil {
		core.WriteResponse(ctx, UserParamsErr, nil)
		return
	}

	_, err := govalidator.ValidateStruct(params)
	if err != nil {
		errno := UserParamsErr
		errno.Message = err.Error()
		core.WriteResponse(ctx, errno, nil)
		return
	}

	_, err = uc.userUsecase.Create(ctx, &params)
	if err != nil {
		core.WriteResponse(ctx, err, nil)
		return
	}

	core.WriteResponse(ctx, errno.OK, nil)
}

func (uc *UserController) UpdateInfo(ctx *gin.Context) {
}

func (uc *UserController) ChangePassword(ctx *gin.Context) {
}
